@@ -24,6 +24,7 @@
 ###################################################################################
 import concurrent
 import configparser
+import http
 import io
 import json
 import os
@@ -44,6 +45,8 @@ import getpass
 import shutil
 import stat
 import urllib.parse
+import http.server
+import socketserver
 
 import logging
 
@@ -68,9 +71,16 @@ import logging
 #  Copy 'run.py' and 'settings.ini' files wherever you want to have your record repository.
 #  Windows let's you double click 'run.py'. Can run from desktop.
 # Magic number 42 is used to connect to all radios found in 'settings.ini'.
-print('0.6.8')
 
-# https://www.scrapehero.com/how-to-fake-and-rotate-user-agents-using-python-3/
+
+version = '0.7.16'
+print(f'{version} now a real Ghetto Blaster with Radio')  # ########## version #############
+
+# - GhettoRecorder.desktop for Snap installation added (shortcut)
+# - added radio, http server redirects to local port
+# - fixed copy file with wrong name
+# - fixed printing match search every time
+
 proxy_support = urllib.request.ProxyHandler({})
 opener = urllib.request.build_opener(proxy_support)
 urllib.request.install_opener(opener)
@@ -88,19 +98,17 @@ urllib3.disable_warnings()
 def switch_dst_dir(container):  # if app runs in a container
 
     global username
+
     print('Hello, ' + username)
-    if container == 'SNAP':
+    if container == 'SNAP':                      # SNAP
         ghetto_folder = '//home//' + username + '//GhettoRecorder'
     else:
         ghetto_folder = '//tmp//GhettoRecorder'  # DOCKER
 
-    # set new radio_base_dir class attribute
     GBase.radio_base_dir = ghetto_folder
 
     source_ini = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'
-    source_script = os.path.dirname(os.path.abspath(__file__)) + '//StartGhetto.sh'
     dst_ini = ghetto_folder + '//settings.ini'
-    dst_script = ghetto_folder + '//StartGhetto.sh'
 
     access_rights = 0o755
     try:
@@ -111,11 +119,8 @@ def switch_dst_dir(container):  # if app runs in a container
     try:
         if not os.path.exists(dst_ini):
             shutil.copyfile(source_ini, dst_ini)
-            shutil.copyfile(source_script, dst_script)
     except FileExistsError:
         pass
-    st = os.stat(dst_script)
-    os.chmod(dst_script, st.st_mode | stat.S_IEXEC)
 
 
 def signal_handler(sig, frame):
@@ -195,6 +200,7 @@ class GIni(GBase):
     search_dict = {'STRINGS': 'Britney Spears ไม่เคยจะจำ Elton AC/DC techno Band feat. mix'}  # only show it is working
     list_items = []
     search_title_keys_list = []  # radio short keys, not start recording all streams, only searched titles
+    content_type = {}  # header info audio/mpeg
 
     def __init__(self):
         super().__init__()
@@ -222,8 +228,8 @@ class GIni(GBase):
             for _ in GIni.list_items:
                 print('\t>> %-20s <<' % _)
 
-            print(' \n Radio stations in your list. ')
-            print(' Please use "Ctrl + C" to stop. ')
+            print(' \n Radio stations in your list.')
+            print(' Please use "Ctrl + C" to stop and get the last file from recording.\n')
         return
 
     @staticmethod
@@ -455,14 +461,13 @@ class GNet(GBase):
         return song_title
 
     @staticmethod
-    def stream_filetype_url(url):
-        headers = GNet.user_agent
-        req = urllib.request.Request(url, headers=headers)
-        with urllib.request.urlopen(req) as response:
-            try:
+    def stream_filetype_url(url, key):
+        try:
+            with urllib.request.urlopen(url, timeout=15) as response:
                 headers = response.getheader('Content-Type')
-            except Exception as ex:
-                print(ex)
+        except Exception as ex:
+            print(ex)
+            return False
 
         content_type = ''
         if headers == 'audio/aacp' or headers == 'application/aacp':
@@ -477,6 +482,15 @@ class GNet(GBase):
             content_type = '.m3u'
         # application/x-winamp-playlist , audio/scpls , audio/x-scpls ,  audio/x-mpegurl
 
+        try:
+            GIni.content_type[key] = headers
+            if len(headers) <= 5:
+                GIni.content_type[key] = 'audio/mpeg'
+            print(f' stream_filetype_url {GIni.content_type[key]}')
+        except Exception as e:
+            print(e)
+            pass
+
         return content_type
 
     @staticmethod
@@ -499,8 +513,10 @@ class GNet(GBase):
             if not GNet.is_ice_json_dict[key]:
                 if not GNet.is_shout_dict[key]:
                     GNet.is_unk_dict[key] = True
-                    req = GNet.http_pool.request('GET', full_url, headers={'Icy-MetaData': '1'}, preload_content=False)
+
                     try:
+                        req = GNet.http_pool.request('GET', full_url, headers={'Icy-MetaData': '1'},
+                                                     preload_content=False)
                         GNet.icy_metadata = int(req.headers['icy-metaint'])
                         print(f' chunk size {GNet.icy_metadata} {key}')
                         # GIni.cost_dict[GIni.cost_current_ini] = GNet.icy_metadata
@@ -508,7 +524,7 @@ class GNet(GBase):
                         # write in dict for no metadata = true
                     except Exception as ex:
                         print(ex)
-                        print(f' {key}: no html title-info, no stream metadata, must be deleted; no record')
+                        print(f' ---> {key}: no html title-info, no stream metadata, must be deleted; no record')
                         GNet.is_no_meta_avail_dict[key] = True
 
             return True
@@ -518,46 +534,20 @@ class GRecorder:
     stream_song_name = '_no-name-record_no_split_'
     path_to_song_dict = {}  # {station : file path}
     current_song_dict = {}  # each thread writes the new title to the station key name {station : title}
-
-    def __init__(self, url, ini_key):
-        self.url = url
-        self.ini_key = ini_key
+    write_command = {}  # recorder head thread set command to copy first file
+    search_pattern_found = {}
+    http_server_port_number = {}  # {key: number}
 
     @staticmethod  # copy keep-alive timeout=3000
-    def thread_pull_song_name(url, ini_key):
-        # print(f' thread_pull_song_name: {ini_key}')
+    def ghetto_recorder_display_titel(url, ini_key):
+        # print(f' ghetto_recorder_display_titel: {ini_key}')
         update_terminal = ''.encode('utf-8')
         stream_song_name = GRecorder.current_song_dict[ini_key]
-        GIni.fail_meta_dict[url] = 'False'
-        headers = {'Connection': 'keep-alive'}
-        # request = GNet.http_pool.request('GET', url, preload_content=False, retries=10, timeout=3000, headers=headers)
+        GIni.fail_meta_dict[url] = 'False'  # message display or not
+        GRecorder.search_pattern_found[ini_key] = False
 
         while not GBase.exit_app:
-            # hd = request.headers  # try keep conn. open
-            # rand = random.randrange(5, 16, 1)  # sleep random servers (5, 29, 1), helped not much in snap container
 
-            # if GIni.srv_param_dict[ini_key + '_is_unknown_server'] == 'True':
-            #     if not GIni.fail_meta_dict[url] == 'True':  # let record go in one file
-            #         pass
-            #         ret = GRecorder.get_metadata_from_stream(url, ini_key)
-            #         stream_song_name = ret
-            #
-            # if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
-            #     ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
-            #     # ice_short_url = GIni.parse_url_simple_url(url)
-            #     # call def for reading json file
-            #     stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url, ini_key)
-            #
-            # if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
-            #     srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
-            #     srv_current = srv + '/currentsong'
-            #     response = GNet.http_pool.request('GET', srv_current, headers=GNet.ctm_usr_agent, preload_content=False)
-            #     stream_song_name = response.data.decode('utf-8')
-            #
-            # if GIni.srv_param_dict[ini_key + '_is_no_meta_data_avail'] == 'True':
-            #     stream_song_name = '_no-name-record_no_split_'
-            #     # print(' _is_no_meta_data_avail ' + stream_song_name)
-            #
             if not len(stream_song_name) <= 2:
                 #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
                 stream_song_name = GRecorder.current_song_dict[ini_key]  # :REVERSE:
@@ -565,8 +555,8 @@ class GRecorder:
                 if not update_terminal == stream_song_name:
                     update_terminal = stream_song_name
                     try:
-                        print('\t\t title on ' + ini_key + ':\t' + update_terminal)
-                        # see what happens. looks like a unit test case
+                        print(f'\t\t {GBase.this_time().split()[1]} title on {ini_key}: \t {update_terminal}')
+                        # see what häppens, góod unìt têst case
                     except UnicodeEncodeError:
                         print('\t\t title on ' + ini_key + ':\t' + 'unicode error (check "$ localectl status")')
                         print('\t\t title on ' + ini_key + ':\t' + 'consider to add additional language support.')
@@ -578,15 +568,18 @@ class GRecorder:
                         pass
             # give recorder command to record, if we got a match from search gui
             #
+
             try:
                 for _ in GIni.search_title_keys_list:
                     if _ == ini_key:
-                        # if some phrases are found in def search_pattern_start_record
+                        # search option was checked,
+                        # call - def search_pattern_start_record, search string match?
                         got_it = GRecorder.search_pattern_start_record(stream_song_name, ini_key)
                         if got_it:
                             GIni.start_stop_recording[ini_key] = 'start'
                         else:
                             GIni.start_stop_recording[ini_key] = 'stop'
+                            GRecorder.search_pattern_found[ini_key] = False
 
             except Exception as ex:
                 print(ex)
@@ -602,11 +595,11 @@ class GRecorder:
 
     @staticmethod
     def search_pattern_start_record(title, ini_key):
-        # search title for strings to see if we should start recording this song/title
+        # search title for strings to see if we should start recording this
         search_list = []  # chop the string
+
         try:
             strings = GIni.search_dict[ini_key]  # dict with user search strings
-            # print(f' strings; {strings} {ini_key}' )
             search_list = strings.encode('utf-8').lower().split(b' ')
         except KeyError as ex:
             pass
@@ -616,8 +609,9 @@ class GRecorder:
             if not len(search_str) <= 1:  # b'' match problem
 
                 if not title.encode('utf-8').lower().find(search_str) == -1:  # find() returns -1, if not found
-                    print(f'<<<< match station: {ini_key} phrase: {search_str}')
-                    # GIni.start_stop_recording[ini_key + '_adv'] = 'start_from_here'
+                    if not GRecorder.search_pattern_found[ini_key]:
+                        print(f'<<<< match station: {ini_key} phrase: {search_str}')
+                        GRecorder.search_pattern_found[ini_key] = True
                     return True
         return False
 
@@ -729,13 +723,17 @@ class GRecorder:
 
     @staticmethod
     def ghetto_recorder_head(directory_save, stream_suffix, radio_short_key):
-        # ORIGINAL
+        # target: tail works without brain until brain call interrupt
         # print(f' ghetto_recorder_head -- {radio_short_key}')
+        sleep(GBase.sleeper)
+        # reader must work (conn. url) for writing the path from title name, can have a loop here
         fresh_song = 'False'
         first_record = True
         main_brake = False
+        GRecorder.write_command[radio_short_key] = False
 
         stream_song_name = GRecorder.current_song_dict[radio_short_key]
+
         while not main_brake:
             i = 0
             while not GBase.exit_app:
@@ -754,20 +752,23 @@ class GRecorder:
 
                     if first_record:
                         fresh_file_path = directory_save + '//' + '_incomplete_' + clean_name + stream_suffix
-                        first_record = False
                         main_brake = True
+                        GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path
+                        GRecorder.write_command[radio_short_key] = True
+
+                        first_record = False
                     else:
                         fresh_file_path = directory_save + '//' + clean_name + stream_suffix
-                    # write new   FILE PATH   for recorder to make next copy
-                    GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # PATH here
-                    # print(f' from head to Tail: {fresh_file_path}')
+                        GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # NEW PATH ... ... ...
+
                     if GBase.exit_app:
                         print(f' exit head {radio_short_key}')
                         break
-                    while not GBase.exit_app:  # inner loop get the stream and write into file
+
+                    while not GBase.exit_app:
 
                         stream_song_name = GRecorder.current_song_dict[radio_short_key]
-                        if not fresh_song == stream_song_name:  # only here break, jump to outer loop
+                        if not fresh_song == stream_song_name:
                             break
 
                         for sec in range(5):
@@ -779,41 +780,64 @@ class GRecorder:
                             sleep(1)
 
     @staticmethod  # org
-    def ghetto_recorder_tail(url, key, path_to_save):
+    def ghetto_recorder_tail(url, key, path_to_save, port_number):
         # shall be as dump as possible
-        # print(f' ghetto_recorder_tail {key} ')
+        while not GBase.exit_app:
+            try:
+                if GRecorder.write_command[key]:
+                    break
+            except KeyError:
+                pass
+            else:
+                sleep(.1)
+
         stream_request_size = io.DEFAULT_BUFFER_SIZE
         old_time = time()
         must_run = False
         brake_loop = False
+        request = ''
         rand_short = random.randrange(4, 13, 3)
         rand_long = random.randrange(61, 90, 1)
 
-        ghetto_recorder = path_to_save + '//__ghetto_recorder.mp3'
-        ghetto_copy = GRecorder.path_to_song_dict[key]  # generates exception, if no dict val
-        sleep(GBase.sleeper + 1)  # await full file path from recorder_head
+        ghetto_recorder = path_to_save + '//__ghetto_recorder' + GIni.srv_param_dict[key + '_file']
+        ghetto_copy = GRecorder.path_to_song_dict[key]  # that's why we have a write command, tail in sync with head
+
+        handler_http_srv_object = MyHttpRequestHandler
+        if port_number:
+            # Create http server thread for local listening from record stream on port: number
+            port = port_number  # dict for {key: port}
+            # handler_http_srv_object = MyHttpRequestHandler
+            my_server = socketserver.TCPServer(("", port), handler_http_srv_object)
+            threading.Thread(target=my_server.serve_forever, name=key, daemon=True).start()
+            handler_http_srv_object.content_type = GIni.content_type[key]  # set type for browser to play
+
         while not GBase.exit_app:
+            # ################################# seek
             while not GBase.exit_app:
                 if GIni.start_stop_recording[key] == 'stop':
                     brake_loop = False
                     must_run = True
-                    sleep(1)
+                    sleep(.1)
                 else:
                     if must_run:
                         print(f'----> start record: {key}')
                     break
+                    # #########################
+            try:
+                request = GNet.http_pool.request('GET', url,
+                                                 headers={'Connection': 'keep-alive'},
+                                                 timeout=3000,
+                                                 preload_content=False)
+            except:
+                pass
 
-            request = GNet.http_pool.request('GET', url,
-                                             headers={'Connection': 'keep-alive'},
-                                             timeout=3000,
-                                             preload_content=False)
             while not GBase.exit_app:
                 with open(ghetto_recorder, 'wb') as record_file:
                     if brake_loop:  # start_stop_recording stop, can change again to start at run.
                         break
 
                     for chunk in request.stream(stream_request_size):  # chunks to file
-
+                        # ################################### seek
                         if GIni.start_stop_recording[key] == 'stop':
                             must_run = False
                             brake_loop = True
@@ -826,26 +850,16 @@ class GRecorder:
                                 print(ex)
                             else:
                                 pass
-                            print(f' ----> stop record: {key}')
+                            print(f'----> stop record: {key}')
                             break
 
+                        # ################################### rec
+
+                        if port_number:
+                            handler_http_srv_object.fifo_http_chunk_queue.put(chunk)  # copy chunk to http server
                         record_file.write(chunk)  # start writing ...
+                        if not ghetto_copy == GRecorder.path_to_song_dict[key]:
 
-                        if time() - old_time > rand_short:
-                            old_time = time()
-                            rh = request.headers
-                            rand_short = random.randrange(4, 12, 2)
-
-                        if time() - old_time > rand_long:
-                            old_time = time()
-                            request = GNet.http_pool.request('GET', url,
-                                                             headers={'Connection': 'keep-alive'},
-                                                             timeout=3000,
-                                                             preload_content=False)
-                            rand_long = random.randrange(61, 90, 1)
-                            rand_short = random.randrange(4, 13, 3)
-                        if not ghetto_copy == GRecorder.path_to_song_dict[key]:  # Compare OLD PATH
-                            ghetto_copy = GRecorder.path_to_song_dict[key]      # SET NEW PATH
                             try:
                                 if os.path.exists(ghetto_copy):
                                     os.remove(ghetto_copy)
@@ -857,19 +871,34 @@ class GRecorder:
                                 record_file.truncate()
                                 record_file.seek(0)
 
+                        ghetto_copy = GRecorder.path_to_song_dict[key]  # SET NEW PATH after copy
+                        # #################################
+                        if time() - old_time > rand_short:
+                            old_time = time()
+                            rh = request.headers
+                            rand_short = random.randrange(4, 12, 2)
+
+                        if time() - old_time > rand_long:
+                            old_time = time()
+                            try:
+                                request = GNet.http_pool.request('GET', url,
+                                                                 headers={'Connection': 'keep-alive'},
+                                                                 timeout=3000,
+                                                                 preload_content=False)
+                            except:
+                                pass
+                            rand_long = random.randrange(61, 90, 1)
+                            rand_short = random.randrange(4, 13, 3)
                         if not chunk:
                             request = GNet.http_pool.request('GET', url,
                                                              headers={'Connection': 'keep-alive'},
                                                              timeout=3000,
                                                              preload_content=False)
-                            rh = request.headers
+                            rh = request.headers  # into oblivion
                             break
                         if GBase.exit_app:
                             sleep(2)  # wait for last file name _incomplete_.....
                             try:
-                                if os.path.exists(path_to_save + '//__ghetto_trash.mp3'):
-                                    os.remove(path_to_save + '//__ghetto_trash.mp3')
-
                                 record_file.flush()
                                 ghetto_copy = GRecorder.path_to_song_dict[key]
                                 print(f' {key} copy last file to folder (marked: _incomplete_): '
@@ -880,11 +909,10 @@ class GRecorder:
                                 if os.path.exists(ghetto_recorder):
                                     os.remove(ghetto_recorder)
                             except UnicodeEncodeError:
-                                print(f' {key} copy last file to folder (marked: _incomplete_):')
+                                print(f' {key} copy last file (marked: _incomplete_):')
                             except Exception as ex:
                                 print(ex)
                             break
-
                 if GBase.exit_app:
                     break
             if GBase.exit_app:
@@ -894,30 +922,40 @@ class GRecorder:
     @staticmethod
     def get_metadata_from_stream_loop(url, key):
         # urllib3 request.stream with headers={'Icy-MetaData': '1'} , keep open (mdr, swr, br5) test
+        rh = ''
+        response = ''
+        request = ''
         old_time = time()
+        rand_short = random.randrange(4, 12, 2)
+        rand_long = random.randrange(50, 80, 1)
 
         while not GBase.exit_app:
-
-            response = GNet.http_pool.request('GET', url,
-                                              headers={'Connection': 'keep-alive'},
-                                              timeout=3000, preload_content=False)
+            try:
+                response = GNet.http_pool.request('GET', url,
+                                                  headers={'Connection': 'keep-alive'},
+                                                  timeout=3000, preload_content=False)
+            except:
+                pass
 
             while not GBase.exit_app:
-                rand_short = random.randrange(4, 12, 2)
-                rand_long = random.randrange(50, 80, 1)
 
                 if time() - old_time > rand_short:
                     old_time = time()
                     rh = response.headers  # keep conn. alive
+                    rand_short = random.randrange(4, 12, 2)
                     # print(response.headers)
 
                 if time() - old_time > rand_long:  # new connection
                     old_time = time()
+                    rand_long = random.randrange(50, 80, 1)
                     break
+                try:
+                    request = GNet.http_pool.request('GET', url,
+                                                     headers={'Icy-MetaData': '1'},
+                                                     preload_content=False)
+                except:
+                    pass
 
-                request = GNet.http_pool.request('GET', url,
-                                                 headers={'Icy-MetaData': '1'},
-                                                 timeout=3000, preload_content=False)
                 try:
                     icy_metadata = request.headers['icy-metaint']
                     icy_metadata = int(icy_metadata)
@@ -925,7 +963,6 @@ class GRecorder:
                     request.read(icy_metadata)
                     chunk_1b = request.read(1)
                     chunk_1b = ord(chunk_1b)
-                    # \x03 hex value, was div. by 4; hex C (12d) * hex 4 = 30 48d; hex 3 (3d) * 16d = 48d;
                     read_bytes = chunk_1b * 16
                     read_bytes = int(read_bytes)
                     metadata_content = request.read(read_bytes)
@@ -933,7 +970,7 @@ class GRecorder:
                     title_info = metadata_content.split(";")
                     title_info = title_info[0].split("=")
                     title_info = GBase.remove_special_chars(title_info[1])
-                    request.release_conn()  # REQUEST not response
+                    request.release_conn()  # REQUEST not response, a one shot anyway
                     # print(f' -> {key} + {title_info}')
                 except Exception as ex:
                     print(ex)
@@ -1126,6 +1163,36 @@ class GRecorder:
                 return False
 
 
+class MyHttpRequestHandler(http.server.SimpleHTTPRequestHandler):
+
+    fifo_http_chunk_queue = queue.Queue(maxsize=10)  # (buffer) we are 10 chunks behind record thread
+    content_type = "audio/mpeg"  # instance, will be overwritten
+    # port 1242 - 1270 free
+
+    def do_GET(self):
+
+        self.send_response(200)
+        self.send_header("Content-type", self.content_type)
+        self.send_header("Cache-Control", "no-cache,no-store")
+        self.send_header("Connection", "keep-alive")
+        self.end_headers()
+        print(f'\t The FIRST radio can be listened to, on local host. (first input)')
+        print(f"\t Stream file type is: {self.content_type}, some browser don't play and try to download.")
+        while not GBase.exit_app:
+            try:
+                self.wfile.write(self.fifo_http_chunk_queue.get())
+            except ConnectionAbortedError:
+                # print(' \tlocal host connection aborted')
+                self.close_connection = True
+                break
+            except Exception as e:
+                print(e)
+                self.close_connection = True
+                break
+            else:
+                pass
+
+
 def step1_collect_stream_server():
     # collect the keys from settings.ini, test if server is alive
     # produce a dictionary with key: short name, value: url
@@ -1221,13 +1288,14 @@ def step2_test_stream_server(ini_keys):
             # print(f"The outcome is {fut.result()} {fut.__dict__}")
 
     for radio in GNet.is_no_meta_avail_dict:
+        print(f' {radio} \t\t url: {GIni.ini_keys[radio]}')
         radio_failed = GNet.is_no_meta_avail_dict[radio]
         if radio_failed:
             del GIni.ini_keys[radio]
             print(f' ---> {radio}: deleted from record list. Check, if server has multiple '
                   f'streams (quality or user limits) {GIni.srv_param_dict[radio]}')
 
-    record(ini_keys)
+    record(ini_keys, False)
     # record(ini_keys, GIni.srv_param_dict)  # this starts record threads and loops at end for shutdown signal
 
 
@@ -1249,11 +1317,15 @@ def print_stream_server():
     print(f'GIni.srv_param_dict {GIni.srv_param_dict}')
 
 
-def record(ini_keys):
+def record(ini_key_val, break_loop):
+    # {blues_uk: http//......}
     signal.signal(signal.SIGTERM, signal_handler)
     signal.signal(signal.SIGINT, signal_handler)
 
-    for ini_key in ini_keys:
+    first_radio = True
+    port_number = 1242  # 1242 - 1270 free ports
+
+    for ini_key in ini_key_val:
         stream_suffix = '.mp3'  # fix for 50%, lol
         try:
             stream_suffix = GIni.srv_param_dict[ini_key + '_file']  # sometimes malformed content-type written?
@@ -1269,12 +1341,24 @@ def record(ini_keys):
 
         url = GIni.ini_keys[ini_key]
         dir_save = GBase.radio_base_dir + '//' + ini_key
-        GBase.pool.submit(GRecorder.thread_pull_song_name, url, ini_key)
+
+        GBase.pool.submit(GRecorder.ghetto_recorder_display_titel, url, ini_key)
         GBase.pool.submit(GRecorder.ghetto_recorder_head, dir_save, stream_suffix, ini_key)
-        GBase.pool.submit(GRecorder.ghetto_recorder_tail, url, ini_key, dir_save)
+        if first_radio:
+            GBase.pool.submit(GRecorder.ghetto_recorder_tail, url, ini_key, dir_save, port_number)  # GUI!!
+            print(f'\n --> Listen to your radio "{ini_key}": http://localhost:{port_number} from just recorded stream.')
+            print(f'\t\t Bookmark in your browser.\n')
+        else:
+            GBase.pool.submit(GRecorder.ghetto_recorder_tail, url, ini_key, dir_save, False)
         GBase.pool.submit(GRecorder.get_metadata_from_stream_loop, url, ini_key)
 
+        # GRecorder.http_server_port_number[ini_key] = port_number
+        port_number += 10
+        first_radio = False
+
     while not GBase.exit_app:
+        if break_loop:
+            break
         sleep(1)
 
 
@@ -1296,13 +1380,16 @@ def test_stream_server(key):
     # each server type property has a dict, { radio name : False/True}, with a small db it would be much cleaner
     # GNet.is_shout_ice_unknown_server(server_port, full_url, key)
     if GNet.is_shout_ice_unknown_server(GIni.srv_param_dict[ini_keys[key]], GIni.srv_param_dict[key], key):
+        # if this server is halfway a radio server
 
         # str_in is input, modify to get more dictionary keys; file type, is shout is ice ...
-        ret = GNet.stream_filetype_url(GIni.srv_param_dict[key])
-
-        GIni.srv_param_dict[key + '_file'] = ret
-        if ret is None or ret == '' or ret == ' ':
-            GIni.srv_param_dict[key + '_file'] = '.mp3'
+        ret = GNet.stream_filetype_url(GIni.srv_param_dict[key], key)
+        if not ret:
+            GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'True'
+        else:
+            GIni.srv_param_dict[key + '_file'] = ret
+            if ret is None or ret == '' or ret == ' ':
+                GIni.srv_param_dict[key + '_file'] = '.mp3'
 
         if GNet.is_shout_dict[key]:
             GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'True'
