@@ -41,7 +41,7 @@ class UIWindow(tk.Tk):
     def __init__(self, radio_dir=None, settings_path=None):
         super().__init__()
         self.list_e = []
-        self.gr_version = '0.7.0'
+        self.gr_version = '0.7.16'
         self.radio_dir = radio_dir
         self.settings_path = settings_path
 
@@ -342,17 +342,13 @@ class UIWindow(tk.Tk):
         for radio in term.GNet.is_no_meta_avail_dict:
             radio_failed = term.GNet.is_no_meta_avail_dict[radio]
             if radio_failed:
-                del term.GIni.ini_keys[radio]
-                print(f' ---> {radio}: deleted from record list. Check, if server has multiple '
+                try:
+                    del term.GIni.ini_keys[radio]
+                    print(f' ---> {radio}: deleted from record list. Check, if server has multiple '
                       f'streams (quality or user limits) {term.GIni.srv_param_dict[radio]}')
+                except KeyError:
+                    pass
 
-        for radio in term.GNet.is_no_meta_avail_dict:
-            print(f' {radio} \t\t url: {term.GIni.ini_keys[radio]}')
-            radio_failed = term.GNet.is_no_meta_avail_dict[radio]
-            if radio_failed:
-                del term.GIni.ini_keys[radio]
-                print(f' ---> {radio}: deleted from record list. Check, if server has multiple '
-                      f'streams (quality or user limits) {term.GIni.srv_param_dict[radio]}')
         # for key in term.GIni.ini_keys:
             # term.GBase.pool.submit(term.record, key)
             # term.GBase.pool.submit(self.start_records, term.GIni.ini_keys)   # ######################################
